#!/bin/ksh 

query_yes_no()
{
   output="$1"
   while true
   do
      echo "$output"
      read yn
      if [ "$yn" = "" -o "$yn" = "y" -o "$yn" = "Y" -o "$yn" = "yes" -o "$yn" = "YES" ]
      then
         return 0
      elif [ "$yn" = "n" -o "$yn" = "N" -o "$yn" = "no" -o "$yn" = "NO" ]
      then
         return 1
      else
         echo "\nExpecting yes or no.\n"
      fi
   done
}

set_source()
{
   while true
   do
      echo "Enter the path to the cdrom mount point: \c"
      read source_dir
      if [ -r $source_dir ] && [ -r $source_dir/collect ]
      then
         export SOURCE=$source_dir
         return 0
      else
         echo "\nI could not find the collect directory in $source_dir"
         echo "Please try again\n"
      fi
   done
}

set_destination()
{
   echo "Enter the destination path: \c"
   read destination_dir
   if [ ! -d $destination_dir ]
   then
      mkdir -p $destination_dir
   fi
   export DESTINATION=$destination_dir
}

copy_files_different_case()
{
   for X in `ls`
   do
      NAME="`echo $X | cut -d '.' -f1`"
      EXTENSION="`echo $X | cut -d '.' -f2`"
      U_NAME="`echo $NAME | tr [:lower:] [:upper:]`"
      L_EXTENSION="`echo $EXTENSION | tr [:upper:] [:lower:]`"
      cp $X $MOVETO/$U_NAME.$L_EXTENSION
   done
   return
}

process_collection()
{
   # cd into the current collection directory
   # sanity check, must have bulk,collect,pdf
   cd $SOURCE/collect/$FILE
   if [ ! -d bulk ] || [ ! -d collect ] || [ ! -d pdf ]
   then
      echo "\nCannot find necessary structure. Exiting\n"
      exit -1
   fi

   if [ ! -d $DESTINATION/collect/$U_COLLNAME/collect ]
   then
      mkdir $DESTINATION/collect/$U_COLLNAME/collect
   fi
   cp -r collect $DESTINATION/collect/$U_COLLNAME/.

   cd bulk
   if [ ! -d $DESTINATION/collect/$U_COLLNAME/bulk ]
   then 
      mkdir $DESTINATION/collect/$U_COLLNAME/bulk
   fi
   MOVETO=$DESTINATION/collect/$U_COLLNAME/bulk
   copy_files_different_case
   cd ..

   cd pdf
   if [ ! -d $DESTINATION/collect/$U_COLLNAME/pdf ]
   then
      mkdir $DESTINATION/collect/$U_COLLNAME/pdf
   fi
   MOVETO=$DESTINATION/collect/$U_COLLNAME/pdf
   copy_files_different_case
   
   return
}

# Entry point of the script
umask 002

set_source
set_destination

echo "\nThe collection at $SOURCE will be copied to $DESTINATION"

if query_yes_no "Continue [y]: \c"
then
   echo "OK"
else
   exit -1
fi

# the collect directory is always there in lower case, so make it if necessary.
if [ ! -d $DESTINATION/collect ]
then
   mkdir -p $DESTINATION/collect
fi

cp -r $SOURCE/* $DESTINATION/.
