/*
**  HyperHelp Viewer
**
**  Bristol Technology Incorporated
**  241 Ethan Allen Highway, Ridgefield, Connecticut 06877
**
**  Copyright (c) 1990-1995 Bristol Technology Inc.
**  Property of Bristol Technology Inc.
**  All rights reserved.
**
**           ***      BTI CONFIDENTIAL INFORMATION      ***
**
**       THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF BRISTOL
**       TECHNOLOGY INC.
**
**       The copyright notice above does not evidence any actual
**       or intended publication of such source code.
**
**  File:  baggage.h
**
**  Author(s):
**     Mark LoSacco
**
**  Description:
**  The functions prototyped in this file implement the Baggage Access API.
**  These are public functions which an external dll routine
**  may use to access baggage data from a HyperHelp help file.
**
**  Change Log:   (Date, Initials, Description)
**  1/12/94        MSL     Created
**
**/

/* Baggage Handle */
typedef long BHND;
 
/********************************************************************/
/*
 * Function: HHBaggageOpen
 * Purpose:  Open the baggage file szBaggageName located in the
 *           help file szFilename with access mode pMode.
 * Returns:  Handle to baggage on success.
 *           NULL (0) on failure.
 */
BHND HHBaggageOpen(char *szFilename, char *szBaggageName, char *pMode);

/********************************************************************/
/*
 * Function: HHBaggageClose
 * Purpose:  Close an open Baggage file and free up memory
 *           associated with it.
 * Returns:  Nothing
 */
void HHBaggageClose(BHND BaggageHnd);

/********************************************************************/
/*
 * Function: HHBaggageRead
 * Purpose:  Read nBytes into pData from the Baggage file
 *           referenced by BaggageHnd.
 * Returns:  Number of bytes read on success.
 *           -1 on failure.
 */
long HHBaggageRead(BHND BaggageHnd, char *pData, long nBytes);

/********************************************************************/
/*
 * Function: HHBaggageTell
 * Purpose:  Return current offset within the Baggage file
 *           referenced by BaggageHnd.
 * Returns:  Current offset on success.
 *           -1 on failure.
 */
long HHBaggageTell(BHND BaggageHnd);

/********************************************************************/
/*
 * Function: HHBaggageSeek
 * Purpose:  Set current position within the Baggage file
 *           referenced by BaggageHnd to nOffset from the
 *           Origin.  Origin may be SEEK_SET, SEEK_CUR, or
 *           SEEK_END.
 * Returns:  New position offset in bytes from beginning of
 *           the Baggage file if success.
 *           -1 if failure.
 * Notes:    This function will not return a position past
 *           the end of the file.  If a seek entails exceeding
 *           the eof, failure is returned.
 */
long HHBaggageSeek(BHND BaggageHnd, long nOffset, int Origin);

/********************************************************************/
/*
 * Function: HHBaggageEof
 * Purpose:  Determine whether the position in the Baggage
 *           file referenced by BaggageHnd is at the end.
 * Returns:  1 if at end of file.
 *           0 if not at end of file.
 *           -1 on error.
 */
int HHBaggageEof(BHND BaggageHnd);

/********************************************************************/
/*
 * Function: HHBaggageSize
 * Purpose:  Determine the size in bytes of the Baggage file
 *           referenced by BaggageHnd.
 * Returns:  Number of bytes on success.
 *           -1 on failure.
 */
long HHBaggageSize(BHND BaggageHnd);


