/*
**  HyperHelp Viewer
**
**  Bristol Technology Incorporated
**  241 Ethan Allen Highway, Ridgefield, Connecticut 06877
**
**  Copyright (c) 1990-1995 Bristol Technology Inc.
**  Property of Bristol Technology Inc.
**  All rights reserved.
**
**           ***      BTI CONFIDENTIAL INFORMATION      ***
**
**       THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF BRISTOL
**       TECHNOLOGY INC.
**
**       The copyright notice above does not evidence any actual
**       or intended publication of such source code.
**
**  File:  edsapi.h
**
**  Author(s):
**     Mark LoSacco
**
**  Description:
**  The functions prototyped in this file implement the External
**  Data Source API.
**  These are public functions which an external dll routine
**  may use to add data to a help topic dynamically at run time.
**
**  Change Log:   (Date, Initials, Description)
**  2/20/94        MSL     Created
**
**/
#include <X11/Intrinsic.h>

/* Property Types */
#define P_Style         1  /* Not available through EDS API     */ 
#define P_FontSize      2  /* Integer font point size           */
#define P_Font          3  /* Font Family (see F_XXX defines)   */ 
#define P_Bold          4  /* 1 = On, 2 = Off                   */
#define P_Italic        5  /* 1 = On, 2 = Off                   */
#define P_Justification 6  /* Justification (see J_XXX defines) */
#define P_Underline     7  /* 1 = On, 2 = Off                   */
#define P_TabWidth      8  /* Tab width in inches               */
#define P_ForeColor     9  /* Foreground RGB value              */
#define P_BackColor     10 /* Background RGB value              */
#define P_LeftIndent    11 /* Left Indentation in inches        */
#define P_FirstIndent   12 /* First Line indentation in inches  */
#define P_SpaceBefore   13 /* Space before paragraph in inches  */
#define P_BorderTop     14 /* Not available through EDS API     */
#define P_BorderBottom  15 /* Not available through EDS API     */
#define P_BorderLeft    16 /* Not available through EDS API     */
#define P_BorderRight   17 /* Not available through EDS API     */
#define P_SpaceAfter    18 /* Space after paragraph in inches   */
#define P_SpaceBetween  19 /* Space between lines in inches     */
#define P_DoubleUnderline 20 
#define P_Strike	21
#define P_Overline	22
#define P_SupScript	23
#define P_SubScript	24

#define P_MaxProperty	24
 
/* Justification Values */
#define J_LEFT          1  /* Left justified                */
#define J_RIGHT         2  /* Right justified               */
#define J_CENTER        3  /* Centered                      */
#define J_JUSTIFIED     4  /* Not available through EDS API */

/* Font Family Values */
#define F_HELV		1  /* Helvetica              */
#define F_COURIER	2  /* Courier                */
#define F_TIMES		3  /* Times Roman            */
#define F_SYMBOL	4  /* Symbol                 */
#define F_NEWCENT	5  /* New Century Schoolbook */

/* EDS Handle */
typedef long EDSHND;

/********************************************************************/
/*
 * Function: HHStartParagraph 
 * Purpose:  Start a new paragraph of text applying the appropriate 
 *           paragraph formatting.
 * Returns:  1 on Success 
 *           0 on Failure.
 */
int HHStartParagraph(EDSHND DataHnd);

/********************************************************************/
/*
 * Function: HHAddText
 * Purpose:  Add text szText to the current topic at the current
 *           position using the current formatting properties. 
 * Returns:  1 on Success 
 *           0 on Failure.
 */
int HHAddText(char *szText, EDSHND DataHnd);

/********************************************************************/
/*
 * Function: HHAddTopicMacro
 * Purpose:  Add topic macro szMacro to the list of macros
 *           executed each time the current topic is
 *           displayed.
 * Returns:  1 on Success 
 *           0 on Failure.
 */
int HHAddTopicMacro(char *szMacro, EDSHND DataHnd);

/********************************************************************/
/*
 * Function: HHAddCommand
 * Purpose:  Add hyperlinked text szText to the current topic
 *           at the current position.  The macro string
 *           szMacro is executed when the text is clicked.
 * Returns:  1 on Success 
 *           0 on Failure.
 * Notes:    Any of the supported HyperHelp macros may be used
 *           as szMacro.  Multiple macros must be semi-colon
 *           delimited.
 */
int HHAddCommand(char *szText, char *szMacro, EDSHND DataHnd);

/********************************************************************/
/*
 * Function: HHAddSoftReturn
 * Purpose:  Add linefeed to the current topic at the current
 *           location without starting a new paragraph.
 * Returns:  1 on Success 
 *           0 on Failure.
 */
int HHAddSoftReturn(EDSHND DataHnd);

/********************************************************************/
/*
 * Function: HHAddPropertyChange
 * Purpose:  Change current formatting property nPropType to
 *           value fPropValue.
 * Returns:  1 on Success 
 *           0 on Failure.
 * Notes:    nPropType must be one of the P_XXX types defined
 *           in edsapi.h.
 *           fPropValue is dependent upon nPropType.
 */
int HHAddPropertyChange(int nPropType, float fPropValue,
                        EDSHND DataHnd);

/********************************************************************/
/*
 * Function: HHAddPixmap
 * Purpose:  Add a pixmap to the current topic at the current location.
 * Returns:  1 on Success
 *           0 on Failure.
 *
 *
 */
int HHAddPixmap(Pixmap pixmap,
		Dimension nWidth,
		Dimension nHeight,
		char nJustification,
		EDSHND DataHnd);
